@@ -19,11 +19,15 @@
  */
 
 
-#include <stdio.h>
-#include <string.h>
-#include <stdlib.h>
 #include "dact.h"
 #include "buffer.h"
+#include <stdio.h>
+#ifdef HAVE_STRING_H
+#include <string.h>
+#endif
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
 
 uint32_t bit_buffer_data=0;
 int bit_buffer_location=0;
