@@ -2,19 +2,37 @@
 	Serpent Cipher from libmcrypt
 */
 #include "dact.h"
-#ifdef HAVE_LIBMCRYPT
+#include "cipher_serpent.h"
+#ifdef HAVE_MCRYPT
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
+#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
+#endif
+#ifdef HAVE_STRING_H
 #include <string.h>
+#endif
+#ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
+#endif
+#ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
+#endif
 #include <fcntl.h>
 #include <stdio.h>
 #include <time.h>
-#include "cipher_serpent.h"
 #include "parse.h"
 #include "ui.h"
 
+#if defined(USE_MODULES) && defined(AS_MODULE)
+#include "module.h"
+uint32_t DC_NUM=2;
+uint32_t DC_TYPE=DACT_MOD_TYPE_ENC;
+void *DC_ALGO=cipher_serpent;
+char *DC_NAME="serpent (MOD)";
+#endif
+
 MCRYPT mcrypt_tdid;
 
 int cipher_serpent(const char *inblock, char *outblock, const int blksize, char *key, const int mode) {
