@@ -23,8 +23,12 @@
 #include "comp_delta.h"
 #include "buffer.h"
 #include <stdio.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
+#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
+#endif
 
 
 
@@ -38,7 +42,9 @@
 */
 
 #if defined(USE_MODULES) && defined(AS_MODULE)
-int DC_NUM=2;
+#include "module.h"
+uint32_t DC_NUM=2;
+uint32_t DC_TYPE=DACT_MOD_TYPE_COMP;
 void *DC_ALGO=comp_delta_algo;
 char *DC_NAME="Delta Compression (MOD)";
 #endif
@@ -79,14 +85,10 @@ int comp_delta_compress(unsigned char *prev_block, unsigned char *curr_block, ch
 			bit_buffer_write(CurrByte,9);
 		}
 		y=bit_buffer_size();
-		if (y>=8 && y!=16) {
+		while (y>=8 && y!=16) {
 			out_block[++x]=bit_buffer_read(8);
+			if (x>=DACT_BLK_SIZE*2) return(-1);
 		}
-		if (y==16) {
-			out_block[++x]=bit_buffer_read(8);
-			out_block[++x]=bit_buffer_read(8);
-		}
-		if (x>=DACT_BLK_SIZE*2) return(-1);
 		
 	}
 	y=bit_buffer_size();
