@@ -29,11 +29,21 @@
 
 #include "dact.h"
 #include "comp_mzlib2.h"
+
+#ifdef HAVE_LIBZ
+#ifdef HAVE_ZLIB_H
 #include <zlib.h>
+#endif
 #include <stdio.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
+#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
+#endif
+#ifdef HAVE_STRING_H
 #include <string.h>
+#endif
 #include "sort.h"
 #define SQRD_256 65536
 
@@ -47,11 +57,14 @@
 	blk_size	- Size of prev_block and curr_block.
 */
 #if defined(USE_MODULES) && defined(AS_MODULE)
-int DC_NUM=8;
+#include "module.h"
+uint32_t DC_NUM=8;
+uint32_t DC_TYPE=DACT_MOD_TYPE_COMP;
 void *DC_ALGO=comp_mzlib2_algo;
 char *DC_NAME="Second Modified Zlib Compression (MOD)";
 #endif
 
+#ifdef DEBUG
 int comp_mzlib2_algo(int mode, unsigned char *prev_block, unsigned char *curr_block, char *out_block, int blk_size) {
 	switch(mode) {
 		case DACT_MODE_COMPR:
@@ -65,6 +78,11 @@ int comp_mzlib2_algo(int mode, unsigned char *prev_block, unsigned char *curr_bl
 			return(-1);
 	}
 }
+#else
+int comp_mzlib2_algo(int mode, unsigned char *prev_block, unsigned char *curr_block, char *out_block, int blk_size) {
+	return(-1);
+}
+#endif
 
 int comp_mzlib2_compress(unsigned char *prev_block, unsigned char *curr_block, char *out_block, int blk_size) {
 	unsigned long dest_size;
@@ -78,7 +96,6 @@ int comp_mzlib2_compress(unsigned char *prev_block, unsigned char *curr_block, c
 	for (i=0;i<blk_size;i+=2) {
 		x=(curr_block[i]<<8)|(curr_block[i+1]);
 		freq[x]++;
-SPOTVAR_NUM(freq[x]);
 		if (freq[x]==0xffff) return(-1);
 	}
 
@@ -90,7 +107,6 @@ SPOTVAR_NUM(freq[x]);
 			out_block[m++]=freq[i]>>24;
 			out_block[m++]=(freq[i]>>16)&0xff;
 			if (m>=(DACT_BLK_SIZE*2)) {
-SPOTVAR_NUM(m);
 				return(-1);
 			}
 			lookup_table[freq[i]>>16]=i;
@@ -183,3 +199,4 @@ int comp_mzlib2_decompress(unsigned char *prev_block, unsigned char *curr_block,
 	free(tmp_block);
 	return(m);
 }
+#endif
