@@ -33,9 +33,15 @@
 #include "buffer.h"
 #include "sort.h"
 #include <stdio.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
+#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
+#endif
+#ifdef HAVE_STRING_H
 #include <string.h>
+#endif
 
 
 /*
@@ -48,7 +54,9 @@
 */
 
 #if defined(USE_MODULES) && defined(AS_MODULE)
-int DC_NUM=6;
+#include "module.h"
+uint32_t DC_NUM=6;
+uint32_t DC_TYPE=DACT_MOD_TYPE_COMP;
 void *DC_ALGO=comp_snibble_algo;
 char *DC_NAME="Seminibble Encoding (MOD)";
 #endif
@@ -145,7 +153,7 @@ int comp_snibble_decompress(unsigned char *prev_block, unsigned char *curr_block
 			out_block[f]|=(freq[lookup_table[m]]<<( j )) ;
 			j+=2;
 /* Obsficated comes to mind ... */
-			if (j==8) out_block[++f+(j=0)]=0;
+			if (j==8) out_block[++f]=(j=0);
 			m=0;
 			cnt=0;
 		}
