@@ -27,7 +27,7 @@
 #define comp_zlib_algo DACT_FAILED_ALGO
 #else
 int comp_zlib_algo(int mode, unsigned char *prev_block, unsigned char *curr_block, char *out_block, int blk_size);
-int comp_zlib_compress(unsigned char *prev_block, unsigned char *curr_block, char *out_block, int blk_size);
+int comp_zlib_compress(unsigned char *prev_block, unsigned char *curr_block, unsigned char *out_block, int blk_size);
 int comp_zlib_decompress(unsigned char *prev_block, unsigned char *curr_block, char *out_block, int blk_size);
 #endif
 #endif
