@@ -18,17 +18,29 @@
  *      email: dact@rkeene.org
  */
 
+#include "dact.h"
 #define __DACT_C
 #include <fcntl.h>
 #include <stdio.h>
 #include <ctype.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
+#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
+#endif
+#ifdef HAVE_STRING_H
 #include <string.h>
+#endif
+#ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
+#endif
+#ifdef HAVE_SYS_WAIT_H
 #include <sys/wait.h>
+#endif
+#ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
-#include "dact.h"
+#endif
 #include "parse.h"
 #include "dendian.h"
 #include "crc.h"
@@ -49,9 +61,8 @@
 
 
 int print_help(int argc, char **argv);
-void dact_loadconfig(const char *path, char *options);
-int dact_upgrade(const char *options, uint32_t *crcs);
-uint32_t is_latest (const char *options);
+void dact_config_loadfile(const char *path, char *options);
+int dact_config_execute(const char *cmd, char *options);
 int dact_shutdown(int retval);
 char *dact_getoutfilename(const char *orig, const int mode);
 uint32_t dact_process_other(int src, const int dest, const uint32_t magic, const char *options);
@@ -63,9 +74,10 @@ int main(int argc, char **argv);
 extern char *optarg;
 extern int optind, opterr, optopt;
 uint32_t DACT_BLK_SIZE=0;
+char dact_nonetwork=0;
 
 int print_help(int argc, char **argv) {
-	printf("DACT %i.%i.%i-%s by Netfuel Design. <dact@rkeene.org>\n", DACT_VER_MAJOR, DACT_VER_MINOR, DACT_VER_REVISION, DACT_VER_SUB);
+	printf("DACT %i.%i.%i-%s by Keene Enterprises <dact@rkeene.org>\n", DACT_VER_MAJOR, DACT_VER_MINOR, DACT_VER_REVISION, DACT_VER_SUB);
 	printf("usage: %s [options...] [file ...]\n",argv[0]);
 
 	printf("Options:\n");
@@ -76,6 +88,7 @@ int print_help(int argc, char **argv) {
 	printf("  -v          Increase verbosity.\n");
 	printf("  -l          List available algorithms.\n");
 	printf("  -n          Toggle use of CRCs.\n");
+	printf("  -i          Use stdin to read information from instead of /dev/tty.\n");
 #ifndef DACT_UNSAFE
 	printf("  -C          Complain when compression errors occur.\n");
 #endif
@@ -85,12 +98,14 @@ int print_help(int argc, char **argv) {
 	printf("  -h          Give this help.\n");
 	printf("  -V          Display DACT version (%i.%i.%i-%s).\n", DACT_VER_MAJOR, DACT_VER_MINOR, DACT_VER_REVISION, DACT_VER_SUB);
 	printf("  -N          Upgrade DACT.\n");
+	printf("  -a          Upgrade DACT modules.\n");
 	printf("  -b NN       Use a block size of NN bytes for compression.\n");
 	printf("  -e NN       Exclude algorithm NN from being used.\n");
 	printf("  -m CONF     Load config file CONF.\n");
 	printf("  -o FILE     Send output to FILE.\n");
 	printf("  -u URL      Specify download location as URL.\n");
 	printf("  -p URL      Parse URL and print results, then exit.\n");
+	printf("  -M COMMAND  Execute COMMAND as if it had appeared in a config file.\n");
 	printf("  -D DESC     Specify a description of DESC.\n");
 	printf("  -I NN       Use ONLY 2 algorithms, NN and 0.\n");
 	printf("  -U FILE     Use FILE to select download location.\n");
@@ -102,78 +117,214 @@ int print_help(int argc, char **argv) {
 int dact_blksize_calc(int fsize) {
 	if (fsize==0) return(DACT_BLK_SIZE_DEF);
 	if (fsize<(204800)) {
-		return(fsize);
+		return(fsize+5);
 	}
 	return(((int) ((((float) fsize)/102400.0)-(0.9999999)))*65535);
 }
 
-void dact_loadconfig(const char *path, char *options) {
+int dact_config_execute(const char *cmd, char *options) {
 	char *line=NULL, *line_s, *item_buf[4]={NULL, NULL, NULL, NULL};
 	int i;
-	FILE *cfd;
 
-	line_s=line=malloc(512);
-	if ((cfd=fopen(path,"r"))==NULL) return;
-	while (!feof(cfd)) {
-		line=line_s;
-		fgets(line, 511, cfd);
-		if (line[0]=='#') continue;
-		i=(strlen(line)-1);
-		if (line[i]=='\n') line[i]=0;
-		for (i=0;i<4;i++) item_buf[i]=NULL;
-		for (i=0;i<4;i++) {
-			if ((item_buf[i]=strsep(&line, "\t "))==NULL)  break;
-			if (item_buf[i][0]==0) i--;
-		}
-		if (item_buf[0]==NULL || item_buf[1]==NULL) continue;
+	line_s=line=strdup(cmd);
+	if (line[0]=='#') return(0);
+	while (line[strlen(line)-1]<32) line[strlen(line)-1]='\0';
+	for (i=0;i<4;i++) item_buf[i]=NULL;
+	for (i=0;i<4;i++) {
+		if ((item_buf[i]=strsep(&line, "\t "))==NULL)  break;
+		if (item_buf[i][0]==0) i--;
+	}
+	if (item_buf[0]==NULL || item_buf[1]==NULL) return(0); /* This means all commands must have arguments. */
 
-		switch (ELFCRC(0, item_buf[0], strlen(item_buf[0]))) {
-			case 164209419: /* binary_check */
-				options[DACT_OPT_BINCHK]=!!strcmp(item_buf[1],"off");
-				break;
-			case 9456603: /* version_check */
-				options[DACT_OPT_VERCHK]=!!strcmp(item_buf[1],"off");
-				break;
-			case 204349618: /* module_dir */
-				strncpy(moduledirectory,item_buf[1],127);
-				break;
-			case 106360197: /* load_module */
+	switch (ELFCRC(0, item_buf[0], strlen(item_buf[0]))) {
+		case 164209419: /* binary_check */
+			options[DACT_OPT_BINCHK]=!!strcmp(item_buf[1],"off");
+			break;
+		case 9456603: /* version_check */
+			options[DACT_OPT_VERCHK]=!!strcmp(item_buf[1],"off");
+			break;
+		case 204349618: /* module_dir */
+			if ((sizeof(moduledirectory)-strlen(moduledirectory)-1)<=0) break;
+			strncat(moduledirectory,":",sizeof(moduledirectory)-strlen(moduledirectory)-1);
+			strncat(moduledirectory,item_buf[1],sizeof(moduledirectory)-strlen(moduledirectory)-1);
+			break;
+		case 247248556: /* module_load_all */
+			if (strcmp(item_buf[1], "on")==0) {
 				init_modules();
-				load_module(item_buf[1]);
-				break;
-			case 164097267: /* network_access */
+				load_modules_all(options);
+			}
+			break;
+		case 48402100:  /* module_load */
+		case 106360197: /* load_module */
+			init_modules();
+			load_module(item_buf[1], options);
+			break;
+		case 164097267: /* network_access */
 #ifndef NO_NETWORK
-				if (!strcmp(item_buf[1],"off")) {
-					/* This should do something XXX */
-				}
+			dact_nonetwork=!strcmp(item_buf[1],"off");
 #endif
-				break;
-			case 209445231: /* exclude_algo */
-				break;
-			case 168825941: /* block_size */
-				DACT_BLK_SIZE=atoi2(item_buf[1]);
-				break;
-			case 162975987: /* use_urls */
-				options[DACT_OPT_URL]=!!strcmp(item_buf[1],"off");
-				break;
-			case 104235033: /* color_ui */
-				dact_ui_setopt(DACT_UI_OPT_COLOR,!!strcmp(item_buf[1],"off"));
-				break;
+			break;
+		case 209445231: /* exclude_algo */
+			i=(atoi(item_buf[1])&0xff);
+			algorithms[i]=DACT_FAILED_ALGO;
+			break;
+		case 168825941: /* block_size */
+			DACT_BLK_SIZE=atoi2(item_buf[1]);
+			break;
+		case 162975987: /* use_urls */
+			options[DACT_OPT_URL]=!!strcmp(item_buf[1],"off");
+			break;
+		case 104235033: /* color_ui */
+			dact_ui_setopt(DACT_UI_OPT_COLOR,!!strcmp(item_buf[1],"off"));
+			break;
+		case 164800901: /* module_upgrade */
+			if (strcmp(item_buf[1],"on")==0) options[DACT_OPT_UPGRADE]=1;
+			break;
+		case 63160590: /* pass_use_stdin */
+		case 191551086: /* use_stdin */
+			dact_ui_setopt(DACT_UI_OPT_PASSSTDIN, 1);
+			break;
 #ifdef DEBUG
-			default:
-				fprintf(stderr, "Unknown command %s (%i)\n",item_buf[0],ELFCRC(0,item_buf[0],strlen(item_buf[0])));
-				break;
+		default:
+			fprintf(stderr, "Unknown command %s (%i)\n",item_buf[0],ELFCRC(0,item_buf[0],strlen(item_buf[0])));
+			break;
 #endif
-		}
-
 	}
 	free(line_s);
+	return(1);
+}
+
+void dact_config_loadfile(const char *path, char *options) {
+	char *line=NULL;
+	FILE *cfd;
+
+	line=malloc(512);
+	if ((cfd=fopen(path,"r"))==NULL) return;
+	while (!feof(cfd)) {
+		fgets(line, 511, cfd);
+		dact_config_execute(line, options);
+	}
+	free(line);
 	fclose(cfd);
 }
 
+int dact_upgrade_file(const char *name, const char *url_get, const char *url_ver, uint32_t version, const char *dest, const char *options) {
+	int newver, ifd=-1, ofd=-1, x=-1;
+	char *real_dest, *real_url_get, buf[4096];
 
+	if (dest==NULL) {
+		real_dest=parse_url_subst(DACT_BIN_DIR "@@FILE@@.so", name);
+	} else {
+		real_dest=parse_url_subst(dest, name);
+	}
+	real_url_get=parse_url_subst(url_get, name);
+
+	newver=dact_upgrade_file_checkver(name, url_ver, options);
+	if (newver>version) {
+		if (options[DACT_OPT_UPGRADE]) {
+			fprintf(stderr, "There is a new version of %s.  NEW: %i.%i.%i, CURR: %i.%i.%i, fetching...\n", name, DACT_VER_PARTS(newver), DACT_VER_PARTS(version));
+			if ((ifd=open_net(real_url_get, O_RDONLY))>=0) {
+				if ((ofd=open_net(real_dest, O_WRONLY|O_CREAT|O_TRUNC, S_IRWXU))>=0) {
+					while ((x=read(ifd, buf, sizeof(buf)))>0) {
+						write(ofd, buf, x);
+					}
+					close(ifd);
+					close(ofd);
+					if (x<0) PERROR("read");
+				} else {
+					PERROR("open");
+				}
+			} else {
+				PERROR("open");
+			}
+		} else {
+			fprintf(stderr, "There is a new version of %s.  NEW: %i.%i.%i, CURR: %i.%i.%i\n", name, DACT_VER_PARTS(newver), DACT_VER_PARTS(version));
+		}
+	}
+	free(real_dest);
+	free(real_url_get);
+	return(x);
+}
+
+int dact_upgrade_file_checkver(const char *name, const char *url_ver, const char *options) {
+#ifdef CHECK_VERSION
+	uint32_t rem_ver[4]={0, 0, 0, 0};
+	int fd;
+	char rem_vers[3][4]={{0,0,0,0},{0,0,0,0},{0,0,0,0}};
+	char *urlbuf, verbuf[9];
+
+	if (options[DACT_OPT_VERCHK]==0 && options[DACT_OPT_UPGRADE]==0) return(0);
+
+	urlbuf=parse_url_subst(url_ver, name);
+	if ((fd=open_net(urlbuf, O_RDONLY))>=0) {
+		read(fd, &verbuf, 9);
+		close(fd);
+		memcpy(rem_vers[0], verbuf, 3);
+		memcpy(rem_vers[1], verbuf+3, 3);
+		memcpy(rem_vers[2], verbuf+6, 3);
+		rem_ver[0]=atoi(rem_vers[0]);
+		rem_ver[1]=atoi(rem_vers[1]);
+		rem_ver[2]=atoi(rem_vers[2]);
+		rem_ver[3]=(rem_ver[0]<<16)|(rem_ver[1]<<8)|(rem_ver[2]);
+	}
+	free(urlbuf);
+	return(rem_ver[3]);
+#else
+	return(0);
+#endif
+}
+
+int dact_upgrade(char *options) {
+#ifdef DACT_DEBIAN_UPGRADE_PROC
+	char *buf;
+	int status=0, i;
+
+	if (getuid()==0) {
+		buf=dact_ui_getuserinput("Executing `apt-get update' okay [y/N]? ", 5, 0);
+		if (toupper(buf[0])!='Y') {
+			PRINTERR("Failed to upgrade DACT.");
+			free(buf);
+			return(-1);
+		}
+		free(buf);
+
+		if (fork()==0) {
+			execl("/usr/bin/apt-get","apt-get","update",NULL);
+			return(-1);  /* Couldn't run binary. */
+		} else {
+			i=wait(&status);
+			if (WIFEXITED(status)) {
+				if (WEXITSTATUS(status)) {
+					PRINTERR("Failed to run `apt-get update'");
+					return(-1);
+				}
+			}
+		}
+
+		if (fork()==0) {
+			execl("/usr/bin/apt-get","apt-get","install","dact",NULL);
+			return(-1);  /* Couldn't run binary. */
+		} else {
+			i=wait(&status);
+			if (WIFEXITED(status)) {
+				if (WEXITSTATUS(status)) {
+					PRINTERR("Failed to run `apt-get install dact'");
+					return(-1);
+				}
+			}
+		}
+
+		return(1);
+	}
+#else
+	options[DACT_OPT_UPGRADE]=1;
+	return(dact_upgrade_file("dact", DACT_BIN_URL, DACT_BIN_URL_VER, DACT_BIN_VER, DACT_BIN, options));
+#endif
+}
+
+#if 0
 int dact_upgrade(const char *options, uint32_t *crcs) {
-	char *urlsubst, *buf;
+	char *urlsubst, *buf, *dact_binfilebuf;
 	char dact_binfile[256];
 	int inFd, outFd;
 	uint32_t i;
@@ -191,6 +342,9 @@ int dact_upgrade(const char *options, uint32_t *crcs) {
 		}
 		free(buf);
 
+#ifdef WIN32
+#error "Windows has no fork(), CreateProcess() instead"
+#endif
 		if (fork()==0) {
 			execl("/usr/bin/apt-get","apt-get","update",NULL);
 			return(-1);  /* Couldn't run binary. */
@@ -248,6 +402,11 @@ int dact_upgrade(const char *options, uint32_t *crcs) {
 	strncpy(dact_binfile,getenv("HOME"),sizeof(dact_binfile)-1);
 	strncat(dact_binfile,"/.dact/",sizeof(dact_binfile)-strlen(dact_binfile)-1);
 	mkdir(dact_binfile, 0755);
+	dact_binfilebuf=parse_url_subst("@@OSNM@@-@@ARCH@@/", "");
+	strncat(dact_binfile,dact_binfilebuf,sizeof(dact_binfile)-strlen(dact_binfile)-1);
+	free(dact_binfilebuf);
+	mkdir(dact_binfile, 0755);
+	
 	strncat(dact_binfile,"dact.bin",sizeof(dact_binfile)-strlen(dact_binfile)-1);
 #endif
 	urlsubst=parse_url_subst("http://www.rkeene.org/projects/rget/rget.cgi?os=@@OSNM@@&arch=@@ARCH@@&project=dact&file=bin&meth=gz","");
@@ -302,7 +461,7 @@ uint32_t is_latest (const char *options) {
 	return(0);
 #endif
 }
-
+#endif
 
 int dact_shutdown(int retval) {
 	unload_modules();
@@ -331,6 +490,8 @@ char *dact_getoutfilename(const char *orig, const int mode) {
 		case DACT_MODE_DECMP:
 			if (strcmp(&orig[strlen(orig)-4],".dct") && \
 				strcmp(&orig[strlen(orig)-4], ".bz2") && \
+				strcmp(&orig[strlen(orig)-5], ".tbz2") && \
+				strcmp(&orig[strlen(orig)-4], ".tgz") && \
 				strcmp(&orig[strlen(orig)-3], ".gz")) {
 				return(NULL);
 			}
@@ -1045,6 +1206,25 @@ XXX: Todo, make this do something...
 			printf("\n");
 		}
 
+		blk_cnt=0;
+		blksize_size=BYTESIZE(DACT_BLK_SIZE);
+		if (options[DACT_OPT_VERB]) {
+			lseek_net(src, DACT_HDR_REG_SIZE+file_extd_size, SEEK_SET);
+			printf("\n");
+			printf("Break down: \n");
+			printf("  Blk | Algo | Size    | Name\n");
+			printf("  ----+------+---------+---------------------------\n");
+			while (1) {
+				if (read(src, &algo, 1)==0) break;
+				if (algo==0xff) break; /* 0xff is reserved for EOF */
+
+				read_de(src, &blksize, blksize_size, sizeof(blksize));
+				lseek_net(src, blksize, SEEK_CUR);
+				printf("  %03i | %03i  | %-7i | %s\n",blk_cnt,algo,blksize,algorithm_names[algo]);
+				blk_cnt++;
+			}
+		}
+
 		printf("\n");
 
 
@@ -1055,10 +1235,11 @@ XXX: Todo, make this do something...
 }
 
 int main(int argc, char **argv) {
-	unsigned char options[15]={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
+	unsigned char options[20]={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
 	signed char opt;
+	struct stat stat_buf;
 	char **in_files, *in_file=NULL, *out_file=NULL;
-	char dact_binfile[256];
+	char dact_binfilebuf[256], *dact_binfile;
 	int filecnt=0;
 	int in_fd, out_fd;
 	int mode=DACT_MODE_COMPR, ciphernum=-1;
@@ -1068,27 +1249,24 @@ int main(int argc, char **argv) {
 
 	dact_ui_init();
 
-	dact_loadconfig(DACT_CONFIG_FILE, options);
-#ifndef GO32
-	strncpy(dact_binfile,getenv("HOME"),sizeof(dact_binfile)-1);
-	strncat(dact_binfile,"/.dact/dact.conf",sizeof(dact_binfile)-strlen(dact_binfile)-1);
-	dact_loadconfig(dact_binfile, options);
-#endif
-/* XXX: hack, to make upgrade work even if DACT_OPT_BINCHK is enabled, we must
+
+/* hack, to make upgrade work even if DACT_OPT_BINCHK is enabled, we must
  *      get the new version before executing it.
  */
 	if (argv[1]!=NULL) {
-		if (!strcmp(argv[1],"-N")) { return(dact_upgrade(options,crcs)); }
+		if (!strcmp(argv[1],"-N")) return(dact_upgrade(options));
+		if (!strcmp(argv[1],"-a")) options[DACT_OPT_UPGRADE]=1;
 	}
 
-	if (options[DACT_OPT_BINCHK]) {
-#ifdef GO32
-		strcpy(dact_binfile,"c:/dact/dact.exe");
-#else
-		strncpy(dact_binfile,getenv("HOME"),sizeof(dact_binfile)-1);
-		strncat(dact_binfile,"/.dact/dact.bin",sizeof(dact_binfile)-strlen(dact_binfile)-1);
+	dact_config_loadfile(DACT_CONFIG_FILE, options);
+#ifndef GO32
+	strncpy(dact_binfilebuf,getenv("HOME"),sizeof(dact_binfilebuf)-1);
+	strncat(dact_binfilebuf,"/.dact/dact.conf",sizeof(dact_binfilebuf)-strlen(dact_binfilebuf)-1);
+	dact_config_loadfile(dact_binfilebuf, options);
 #endif
 
+	if (options[DACT_OPT_BINCHK]) {
+		dact_binfile=parse_url_subst(DACT_BIN, "");
 		if (strcmp(argv[0],dact_binfile)) {
 			if (!access(dact_binfile,X_OK)) {
 				argv[0]=dact_binfile;
@@ -1097,10 +1275,10 @@ int main(int argc, char **argv) {
 		}
 	}
 
-	while ((opt=getopt(argc,argv,"adfsvcnNVHCE:p:I:m:e:lb:hu:U:TPOD:o:"))!=-1) {
+	while ((opt=getopt(argc,argv,"adfsvcnhiNVHCM:E:p:I:m:e:lb:u:U:TPOD:o:"))!=-1) {
 		switch (opt) {
 			case 'a':
-				PRINTERR("This option does nothing.");
+				options[DACT_OPT_UPGRADE]=1;
 				break;
 			case 'd':
 				mode=DACT_MODE_DECMP;
@@ -1111,6 +1289,9 @@ int main(int argc, char **argv) {
 			case 's':
 				mode=DACT_MODE_STAT;
 				break;
+			case 'i':
+				dact_ui_setopt(DACT_UI_OPT_PASSSTDIN, 1);
+				break;
 			case 'c':
 				options[DACT_OPT_STDOUT]=!options[DACT_OPT_STDOUT];
 				break;
@@ -1133,7 +1314,7 @@ int main(int argc, char **argv) {
 				options[DACT_OPT_COMPLN]++;
 				break;
 			case 'm':
-				dact_loadconfig(optarg, options);
+				dact_config_loadfile(optarg, options);
 				break;
 			case 'e':
 				i=(atoi(optarg)&0xff);
@@ -1145,8 +1326,12 @@ int main(int argc, char **argv) {
 			case 'o':
 				out_file=parse_url_subst(optarg, "");
 				break;
+			case 'M':
+				dact_config_execute(optarg, options);
+				break;
 			case 'N':
-				return(dact_upgrade(options,crcs));
+				PRINTERR("The `-N\' option must be the first and only argument passed to dact.");
+				return(0);
 				break;
 			case 'E':
 				strtolower(optarg);
@@ -1154,14 +1339,14 @@ int main(int argc, char **argv) {
 				if (x==hash_fourbyte("list", ' ')) {
 					PRINT_LINE; fprintf(stderr, "dact: Num | Name\n");
 					for (i=0;i<CIPHER_COUNT;i++) {
-						if (ciphers_name[i]!=NULL) {
+						if (ciphers_name[i]!=NULL && ciphers[i]!=DACT_FAILED_ALGO) {
 							PRINT_LINE; fprintf(stderr, "dact: %3i | %s\n",i,ciphers_name[i]);
 						}
 					}
 					return(0);
 				}
 				for (i=0;i<CIPHER_COUNT;i++) {
-					if (ciphers_name[i]!=NULL) {
+					if (ciphers_name[i]!=NULL && ciphers[i]!=DACT_FAILED_ALGO) {
 						if (x==hash_fourbyte(ciphers_name[i], ' ')) {
 							break;
 						}
@@ -1229,10 +1414,10 @@ int main(int argc, char **argv) {
 
 	}
 
-/* Check for a new version of DACT (is_latest() returns immediately if
- * this option is disabled.)
+/* 
+ * Check for a new version of DACT 
  */
-	if ((i=is_latest(options))) {
+	if ((i=dact_upgrade_file_checkver("dact", DACT_BIN_URL_VER, options))>DACT_BIN_VER) {
 		PRINTERR("**+");
 		PRINTERR("**> There is a new version of DACT available.");
 		PRINTERR("**>");
@@ -1260,10 +1445,17 @@ int main(int argc, char **argv) {
 			if (out_file==NULL) out_file=dact_getoutfilename(in_file,mode);
 			if (!strcmp("-",in_file)) {
 				in_fd=STDIN_FILENO;
-			} else if ((in_fd=open_net(in_file,O_RDONLY))==-1) {
-				fprintf(stderr, "dact: Can't open %s.\n",in_file);
-				PERROR("open");
-				continue;
+			} else {
+				stat(in_file, &stat_buf);
+				if (S_ISDIR(stat_buf.st_mode)) {
+					fprintf(stderr, "dact: %s is a directory.",in_file);
+					continue;
+				}
+				if ((in_fd=open_net(in_file,O_RDONLY))==-1) {
+					fprintf(stderr, "dact: Can't open %s.\n",in_file);
+					PERROR("open");
+					continue;
+				}
 			}
 			if (out_file!=NULL) {
 				if (!strcmp("-",out_file)) options[DACT_OPT_STDOUT]=1;
@@ -1275,6 +1467,8 @@ int main(int argc, char **argv) {
  *  DO NOT MAKE DACT SUID ROOT OR YOU WILL BE HACKED
  * 
  * that should be a suffcient warning.
+ * (this is a mere race condition, but a severe warning should prevent
+ *  people from complaining about it to me.)
  *
  */
 				if (access(out_file,F_OK)!=-1 && options[DACT_OPT_FORCE]==0 && options[DACT_OPT_STDOUT]==0) {
@@ -1317,10 +1511,8 @@ int main(int argc, char **argv) {
 			}
 		}
 /* Cleanup */
-	
 		if (out_fd!=-1) close(out_fd);
 		if (in_fd!=-1) close(in_fd);
-
 	} while (in_files[filecnt++]!=NULL);
 
 	return(dact_shutdown(0));
