@@ -21,7 +21,14 @@
 #ifndef _DACT_H
 #define _DACT_H
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
+#ifdef WIN32
+#include "win32.h"
+#endif
+
+#include "comp_fail.h"
 
 #ifndef DACT_CONTACT
 #define DACT_CONTACT "<dact@rkeene.org>"
@@ -33,7 +40,8 @@
 
 #define DACT_VER_MAJOR 0
 #define DACT_VER_MINOR 8
-#define DACT_VER_REVISION 11
+#define DACT_VER_REVISION 31
+#define DACT_VERSION "0.8.31"
 #ifdef DEBUG
 #define DACT_VER_SUB	"dev"
 #else
@@ -64,6 +72,7 @@
 #define DACT_OPT_NOCRC	11
 #define DACT_OPT_NETWORK	12
 #define DACT_OPT_URL	13
+#define DACT_OPT_UPGRADE	14
 
 /* These should only be changed at the risk of breaking files. 
  * (in other words, they should not be changed if possible.
@@ -120,6 +129,7 @@ struct dact_header {
 };
 
 extern uint32_t DACT_BLK_SIZE;
+extern char dact_nonetwork;
 
 #if defined(__FILE__) && defined(__LINE__) && defined(DEBUG)
 #define PRINT_LINE fprintf(stderr, __FILE__ ":%07i: ", __LINE__)
@@ -130,6 +140,7 @@ extern uint32_t DACT_BLK_SIZE;
 #define PERROR(error) { PRINT_LINE; perror("dact: " error); }
 #define PRINTERR(error) { PRINT_LINE; fprintf(stderr, "dact: " error "\n"); }
 
+/* Win32 check here */
 #define SANITY(x, y...) { if ((x(y))<0) { PERROR(#x); return(-1); } }
 #ifdef DEBUG
 #define SHOWVAL(x...) PRINT_LINE; fprintf(stderr, "dact: %s=%li\n", #x, (long) (x))
@@ -155,4 +166,39 @@ extern uint32_t DACT_BLK_SIZE;
 #define RANDOM_DEV "/dev/urandom"
 #endif
 
+#ifndef DACT_VER_PARTS
+#define DACT_VER_PARTS(x) ((x&0xff0000)>>16),((x&0xff00)>>8),(x&0xff)
+#endif
+
+#ifndef DACT_BIN_URL
+#define DACT_BIN_URL "http://www.rkeene.org/projects/compression/dact/@@OSNM@@-@@ARCH@@/dact.bin"
+#endif
+#ifndef DACT_BIN_URL_VER
+#define DACT_BIN_URL_VER "http://www.rkeene.org/projects/compression/dact/@@OSNM@@-@@ARCH@@/dact.ver"
+#endif
+#ifndef DACT_BIN_DIR
+#define DACT_BIN_DIR "@@HOME@@/.dact/@@OSNM@@-@@ARCH@@/"
+#endif
+#ifndef DACT_BIN
+#define DACT_BIN DACT_BIN_DIR "dact.bin"
+#endif
+#define DACT_BIN_VER ((DACT_VER_MAJOR<<16)|(DACT_VER_MINOR<<8)|DACT_VER_REVISION)
+
+
+
+
+int print_help(int argc, char **argv);
+int dact_blksize_calc(int fsize);
+void dact_config_loadfile(const char *path, char *options);
+int dact_config_execute(const char *cmd, char *options);
+int dact_upgrade_file_checkver(const char *name, const char *url_ver, const char *options);
+int dact_upgrade_file(const char *name, const char *url_get, const char *url_ver, uint32_t version, const char *dest, const char *options);
+int dact_shutdown(int retval);
+char *dact_getoutfilename(const char *orig, const int mode);
+uint32_t dact_process_other(int src, const int dest, const uint32_t magic, const char *options);
+uint32_t dact_blk_decompress(char *ret, const char *srcbuf, const uint32_t size, const char *options, const int algo);
+uint32_t dact_blk_compress(char *algo, char *ret, const char *srcbuf, const uint32_t size, const char *options);
+uint32_t dact_process_file(const int src, const int dest, const int mode, const char *options, const char *filename, uint32_t *crcs, int cipher);
+int main(int argc, char **argv);
+
 #endif/*_DACT_H*/
