@@ -22,13 +22,19 @@
 /*
  *	Network related functions
  */
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <stdarg.h>
-#include <unistd.h>
-#include <fcntl.h>
 #include "dact.h"
 #include "net.h"
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
+#ifdef HAVE_SYS_STAT_H
+#include <sys/stat.h>
+#endif
+#include <stdarg.h>
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+#include <fcntl.h>
 
 #ifndef NO_NETWORK
 #include <sys/socket.h>
@@ -43,7 +49,12 @@
 #include "crc.h"
 
 extern int errno;
-char *dact_urls[256];
+struct dact_url_info {
+	char *url;
+	int flags;
+	mode_t mode;
+};
+struct dact_url_info dact_urls[256];
 
 /*
  *	Create a listening port on tcp port PORT
@@ -74,6 +85,8 @@ int createconnection_tcp(char *host, int port) {
 	int sockid;
 	struct hostent *hostinfo;
 	struct sockaddr_in sock;
+
+	if (dact_nonetwork) return(-EPERM);
 	if ((hostinfo=gethostbyname(host))==NULL) {
 #ifdef HAVE_INET_ATON
 		if (!inet_aton(host,&sock.sin_addr))
@@ -82,6 +95,7 @@ int createconnection_tcp(char *host, int port) {
 #endif
 			return(-1);
 	} else {
+/* Should we assign here? */
 		memcpy(&sock.sin_addr.s_addr,hostinfo->h_addr_list[0],hostinfo->h_length);
 	}
 	sock.sin_family=AF_INET;
@@ -104,16 +118,19 @@ int open_net(const char *pathname, int flags, ...) {
 	va_list args;
 	mode_t mode;
 
-	va_start(args, flags);
-	mode = va_arg(args, mode_t);
-	va_end(args);
+        if (flags==O_WRONLY || flags==O_RDWR) {
+		va_start(args, flags);
+		/* This *SHOULD* be `int' here, not mode_t, for some reason. */
+		mode = va_arg(args, int);
+		va_end(args);
+	}
 
 	if (!parse_url(pathname,scheme,username,password,host,&port,file)) {
 		if ((fd=createconnection_tcp(host,port))<0) return(-1);
 
 		switch (ELFCRC(0, scheme, strlen(scheme))) {
 			case 457648: /* http */
-				if ((flags&3)==O_WRONLY || (flags&3)==O_RDWR) {
+				if ((flags&O_WRONLY)==O_WRONLY || (flags&O_RDWR)==O_RDWR) {
 					close(fd);
 					return(-1);
 				}
@@ -166,12 +183,14 @@ int open_net(const char *pathname, int flags, ...) {
 
 				free(read_buf_s);
 
-				if (dact_urls[fd]!=NULL) free(dact_urls[fd]);
-				dact_urls[fd]=strdup(pathname);
+				if (dact_urls[fd].url!=NULL) free(dact_urls[fd].url);
+				dact_urls[fd].url=strdup(pathname);
+				dact_urls[fd].flags=flags;
+				dact_urls[fd].mode=mode;
 				return(fd);
 				break;
 			case 28080: /* ftp */
-				if ((flags&3)==O_RDWR) {
+				if ((flags&O_RDWR)==O_RDWR) {
 					close(fd);
 					return(-1);
 				}
@@ -243,8 +262,8 @@ int open_net(const char *pathname, int flags, ...) {
 								write(fd, "TYPE I\n", 7);
 								break;
 							case 217888:
-								if ((flags&3)==O_RDONLY) write(fd,"RETR ",5);
-								if ((flags&3)==O_WRONLY) write(fd,"STOR ",5);
+								if ((flags&O_RDONLY)==O_RDONLY) write(fd,"RETR ",5);
+								if ((flags&O_WRONLY)==O_WRONLY) write(fd,"STOR ",5);
 								write(fd, file, strlen(file));
 								write(fd, "\n", 1);
 								if ((ftpfd=createconnection_tcp(host,port))<0) {
@@ -254,8 +273,10 @@ int open_net(const char *pathname, int flags, ...) {
 								}
 								break;
 							case 215072: /* 150 */ 
-								if (dact_urls[ftpfd]!=NULL) free(dact_urls[ftpfd]);
-								dact_urls[ftpfd]=strdup(pathname);
+								if (dact_urls[ftpfd].url!=NULL) free(dact_urls[ftpfd].url);
+								dact_urls[ftpfd].url=strdup(pathname);
+								dact_urls[ftpfd].flags=flags;
+								dact_urls[ftpfd].mode=mode;
 								return(ftpfd);
 								break;
 							case 218656: /* 230 */
@@ -279,8 +300,10 @@ int open_net(const char *pathname, int flags, ...) {
 		close(fd);
 	} else {
 		fd=open(pathname,flags,mode);
-		if (dact_urls[fd]!=NULL) free(dact_urls[fd]);
-		dact_urls[fd]=strdup(pathname);
+		if (dact_urls[fd].url!=NULL) free(dact_urls[fd].url);
+		dact_urls[fd].url=strdup(pathname);
+		dact_urls[fd].flags=flags;
+		dact_urls[fd].mode=mode;
 		return(fd);
 	}
 
@@ -295,7 +318,8 @@ off_t lseek_net(int filedes, off_t offset, int whence) {
 	if (S_ISSOCK(file_status.st_mode)) {
 		if (whence==SEEK_END) return(-1);
 		if (offset!=0 || whence!=SEEK_SET) return(-1); /* For now ... */
-		if ((newfd=open_net(dact_urls[filedes],O_RDONLY))<0) return(-1);
+		if (dact_urls[filedes].url==NULL) return(-1);
+		if ((newfd=open_net(dact_urls[filedes].url,dact_urls[filedes].flags,dact_urls[filedes].mode))<0) return(-1);
 		close(filedes);
 		dup2(newfd,filedes);
 		return(0);
