@@ -21,6 +21,12 @@
 #ifndef _DACT_NET_H 
 #define _DACT_NET_H
 #include "dact.h"
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 
 int createlisten(int port);
 void closeconnection(int sockfd);
